import signal
import sys
import time
from concurrent.futures import ThreadPoolExecutor

import pytest
from zibai.multiprocess import MultiProcessManager, ProcessParameters

from .utils import new_console_in_windows


def while_true():
    signal.signal(signal.SIGINT, lambda sig, frame: sys.exit(0))
    signal.signal(signal.SIGTERM, lambda sig, frame: sys.exit(0))
    while True:
        time.sleep(1)


@new_console_in_windows
def test_multiprocess() -> None:
    """
    Ensure that the MultiProcessManager works as expected.
    """
    multi_process_manager = MultiProcessManager(
        2, ProcessParameters(while_true), join_timeout=5
    )
    executor = ThreadPoolExecutor(max_workers=1)
    future = executor.submit(multi_process_manager.mainloop)
    time.sleep(1)
    multi_process_manager.should_exit.set()
    multi_process_manager.terminate_all_quickly()
    future.result()


@new_console_in_windows
def test_multiprocess_sigbreak() -> None:
    """
    Ensure that the SIGBREAK signal is handled as expected.
    """
    multi_process_manager = MultiProcessManager(
        2, ProcessParameters(while_true), join_timeout=5
    )
    executor = ThreadPoolExecutor(max_workers=1)
    future = executor.submit(multi_process_manager.mainloop)
    time.sleep(1)
    multi_process_manager.should_exit.set()
    multi_process_manager.terminate_all()
    future.result()


@pytest.fixture
def multi_process_manager():
    multi_process_manager = MultiProcessManager(
        2, ProcessParameters(while_true), join_timeout=5
    )
    executor = ThreadPoolExecutor(max_workers=1)
    future = executor.submit(multi_process_manager.mainloop)
    time.sleep(1)
    yield multi_process_manager
    multi_process_manager.should_exit.set()
    multi_process_manager.terminate_all_quickly()
    future.result()


@pytest.mark.skipif(not hasattr(signal, "SIGHUP"), reason="platform unsupports SIGHUP")
def test_multiprocess_sighup(multi_process_manager: MultiProcessManager) -> None:
    """
    Ensure that the SIGHUP signal is handled as expected.
    """
    pids = [p.pid for p in multi_process_manager.processes]
    multi_process_manager.signal_queue.append(signal.SIGHUP)
    time.sleep(1)
    assert pids != [p.pid for p in multi_process_manager.processes]


@pytest.mark.skipif(
    not hasattr(signal, "SIGTTIN"), reason="platform unsupports SIGTTIN"
)
def test_multiprocess_sigttin(multi_process_manager: MultiProcessManager) -> None:
    """
    Ensure that the SIGTTIN signal is handled as expected.
    """
    multi_process_manager.signal_queue.append(signal.SIGTTIN)
    time.sleep(1)
    assert len(multi_process_manager.processes) == 3


@pytest.mark.skipif(
    not hasattr(signal, "SIGTTOU"), reason="platform unsupports SIGTTOU"
)
def test_multiprocess_sigttou(multi_process_manager: MultiProcessManager) -> None:
    """
    Ensure that the SIGTTOU signal is handled as expected.
    """
    multi_process_manager.signal_queue.append(signal.SIGTTOU)
    time.sleep(1)
    assert len(multi_process_manager.processes) == 1
    multi_process_manager.signal_queue.append(signal.SIGTTOU)
    time.sleep(1)
    assert len(multi_process_manager.processes) == 1



import functools
import os
from typing import Any, Callable


def new_console_in_windows(test_function: Callable[[], Any]):
    if os.name != "nt":
        return test_function

    @functools.wraps(test_function)
    def new_function():
        import subprocess
        import sys

        return subprocess.check_call(
            [
                sys.executable,
                "-c",
                f"from {test_function.__module__} import {test_function.__name__}; {test_function.__name__}.__wrapped__()",
            ],
            creationflags=subprocess.CREATE_NO_WINDOW,
        )

    return new_function



import os
import random
import socket
import threading
import time

import pytest
from zibai import create_bind_socket


def create_ipv4_socket():
    for _ in range(10):
        port = random.randint(20000, 60000)
        try:
            return create_bind_socket(f"127.0.0.1:{port}")
        except IOError:
            continue


def create_ipv6_socket():
    for _ in range(10):
        port = random.randint(20000, 60000)
        try:
            return create_bind_socket(f"::1:{port}")
        except IOError:
            continue


def create_unix_socket():
    path = "/tmp/test-zibai-server.sock"
    if os.path.exists(path):
        os.remove(path)
    return create_bind_socket(f"unix:{path}")


@pytest.fixture(params=[create_ipv4_socket, create_ipv6_socket, create_unix_socket])
def bind_socket(request):
    if request.param is create_unix_socket:
        if not hasattr(socket, "AF_UNIX"):
            pytest.skip("AF_UNIX is not supported")

    with request.param() as sock:
        yield sock


@pytest.fixture
def socket_and_event(bind_socket):
    exit_event = threading.Event()
    try:
        yield bind_socket, exit_event
    finally:
        exit_event.set()
        time.sleep(0.15)



import os
import sys

from .cli import main, parse_args


def command_line() -> None:
    sys.path.insert(0, os.getcwd())
    main(parse_args(sys.argv[1:]))


if __name__ == "__main__":
    command_line()
